<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\DDD\Repositories;

use Locr\WordPress\Plugin\LocrMaps\DDD\Entities\Icon;
use Locr\WordPress\Plugin\LocrMaps\{TypedArray, TypedSequence};
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\Icon\Id;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the icons.
 */
final class Icons {
	/**
	 * This function adds an icon into the database.
	 *
	 * @param Icon $icon The icon to add.
	 * @throws \Exception If the icon could not be added.
	 */
	public function add( Icon $icon ): void {
		global $wpdb;

		$insert_data   = array(
			'icon_url' => (string) $icon->url,
			'width'    => $icon->size->width,
			'height'   => $icon->size->height,
		);
		$insert_format = array( '%s', '%d', '%d' );
		$parsed_url    = wp_parse_url( (string) $icon->url );
		if ( is_array( $parsed_url ) && isset( $parsed_url['path'] ) ) {
			$pathinfo            = pathinfo( $parsed_url['path'] );
			$insert_data['name'] = $pathinfo['filename'];
			$insert_format[]     = '%s';
		}

		// @phpstan-ignore property.nonObject
		$table_prefix = $wpdb->prefix;
		if ( ! is_string( $table_prefix ) ) {
			$table_prefix = '';
		}
		// @phpstan-ignore method.nonObject
		$insert_result = $wpdb->insert(
			table: $table_prefix . 'locrmaps_icons',
			data: $insert_data,
			format: $insert_format
		);

		if ( false === $insert_result ) {
			throw new \Exception( 'The icon could not be added (db-error).' );
		}
	}

	/**
	 * This function gets all icons.
	 *
	 * @return TypedSequence The icons.
	 */
	public function get_all_icons(): TypedSequence {
		global $wpdb;

		$icons = TypedSequence::for_type( Icon::class );

		// @phpstan-ignore method.nonObject, property.nonObject, encapsedStringPart.nonString
		$db_result = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}locrmaps_icons", ARRAY_A );
		if ( is_array( $db_result ) ) {
			foreach ( $db_result as $row ) {
				$icons->add( Icon::from_db_row( $row ) );
			}
		}

		return $icons;
	}

	/**
	 * This function gets all icons.
	 *
	 * @return TypedArray The icons.
	 */
	public function get_all_icons_as_associative(): TypedArray {
		global $wpdb;

		$icons = TypedArray::for_type( Icon::class );

		// @phpstan-ignore method.nonObject, property.nonObject, encapsedStringPart.nonString
		$db_result = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}locrmaps_icons", ARRAY_A );
		if ( is_array( $db_result ) ) {
			foreach ( $db_result as $row ) {
				$icon                          = Icon::from_db_row( $row );
				$icons[ (string) $icon->name ] = $icon;
			}
		}

		return $icons;
	}

	/**
	 * This function gets an icon by id.
	 *
	 * @param Id $id The id of the icon.
	 */
	public function get_by_id( Id $id ): ?Icon {
		global $wpdb;

		// @phpstan-ignore method.nonObject
		$db_result = $wpdb->get_row(
			// @phpstan-ignore method.nonObject
			$wpdb->prepare(
				// @phpstan-ignore property.nonObject, encapsedStringPart.nonString
				"SELECT * FROM {$wpdb->prefix}locrmaps_icons WHERE id = %s",
				$id->value
			),
			ARRAY_A
		);

		if ( is_null( $db_result ) ) {
			return null;
		}

		return Icon::from_db_row( $db_result );
	}

	/**
	 * This function gets an icon by name.
	 *
	 * @param string $name The name of the icon.
	 */
	public function get_icon_by_name( string $name ): ?Icon {
		global $wpdb;

		// @phpstan-ignore method.nonObject
		$db_result = $wpdb->get_row(
			// @phpstan-ignore method.nonObject
			$wpdb->prepare(
				// @phpstan-ignore property.nonObject, encapsedStringPart.nonString
				"SELECT * FROM {$wpdb->prefix}locrmaps_icons WHERE name = %s",
				$name
			),
			ARRAY_A
		);

		if ( is_null( $db_result ) ) {
			return null;
		}

		return Icon::from_db_row( $db_result );
	}
}
